@@ -473,4 +473,4 @@ class Registration(Endpoint):
                 name=_context.cookie_handler.name["register"], client_id=reg_resp["client_id"],
             )
 
-            return {"response_args": reg_resp, "cookie": _cookie}
+            return {"response_args": reg_resp, "cookie": _cookie, "response_code": 201}
